@@ -591,8 +591,9 @@ class FakeDir(FakeItem):
                anystart=None, anyend=None, depthlimit=None, itemlimit=None,
                beyond=None, first=None, sort=False, sort_reverse=False, sort_key=None,
                include_folders=None, exclude_folders=None, include_files=None,
-               exclude_files=None, regex=False, slash='/', mask=None,
-               formatter=None, sticky_formatter=False, **kwargs):
+               exclude_files=None, regex=False, mask=None,
+               formatter=None, sticky_formatter=False,
+               acceptable_listdir_errors=None, denied_string=' [ACCESS DENIED]', **kwargs):
         '''
 
         Create a folder tree diagram for `self`.  `seedir.fakedir.FakeDir` version of
@@ -700,12 +701,36 @@ class FakeDir(FakeItem):
             will be permanent.  Thus, if arguments are updated when the `formatter`
             is called on a folder, its children will (recursively) inherit
             those new arguments.
-        slash : str, option:
-            **`DeprecationWarning`**: *With addition of `folderend` in v0.3.1,
-            `slash` is to be deprecated in a future version.*
 
-            Slash character to follow folders.  If `'sep'`, uses `os.sep`.  The
-            default is `'/'`.
+        acceptable_listdir_errors : Exception, tuple, or None
+
+            **New in v0.5.0**
+
+            Set errors which, when raised when listing the contents of a folder,
+            do not halt traversal.  This parameter was added to allow
+            folders to be skipped when a `PermissionError` is raised.  For folders
+            which raise an acceptable error, an additional string is added to their
+            entry in the diagram (see `denied_string`).
+
+            Providing one Exception causes only that type of Exception to be ignored.
+            Multiple Exception types can be handled by passing mutple Exceptions
+            as a tuple.  Pass `None` to not allow any Exceptions (in this case,
+            an error is raised).
+
+            Exception types which are not provided will still be raised.
+
+            This parameter is mostly meaningless for FakeDir objects, except
+            for testing purposes.
+
+        denied_string : str
+
+            **New in v0.5.0**
+
+            String tag to signify that a folder was not able to be traversed
+            due to one of the `acceptable_listdir_errors` being raised.  This
+            is a string added after the folder name (and `folderend`) strings.
+            The default is `" [ACCESS DENIED]"`.
+
         **kwargs : str
             Specific tokens to use for creating the file tree diagram.  The tokens
             use by each builtin style can be seen with `seedir.printing.get_styleargs()`.
@@ -730,6 +755,12 @@ class FakeDir(FakeItem):
             affected by the indent parameter.  The `uniform`, `anystart`, and
             `anyend` parameters can be used to affect multiple style tokens.
 
+        Notes
+        -------
+
+        The parameter `slash` was deprecated in 0.5.0.  Pass `folderend` as
+        an additional keyword argument instead.
+
         Returns
         -------
         s (str) or None
@@ -760,9 +791,12 @@ class FakeDir(FakeItem):
                     mask=mask,
                     formatter=formatter,
                     sticky_formatter=sticky_formatter,
+                    acceptable_listdir_errors=acceptable_listdir_errors,
+                    denied_string=denied_string,
                     **kwargs)
 
-        return FakeDirStructure(self, **args)
+        FDS = FakeDirStructure()
+        return FDS(self, **args)
 
     def set_child_depths(self):
         '''Recursively set depths of `self` and its children.
@@ -1129,26 +1163,27 @@ def recursive_add_fakes(path, parent, depth=0, depthlimit=None,
     None.
 
     '''
+    RDS = RealDirStructure()
     if depthlimit is not None and depth >= depthlimit:
         return
     depth +=1
     listdir = listdir_fullpath(path)
     if sort or first is not None:
-        listdir = RealDirStructure.sort_dir(listdir, first=first,
-                                            sort_reverse=sort_reverse, sort_key=sort_key)
+        listdir = RDS.sort_dir(listdir, first=first,
+                               sort_reverse=sort_reverse, sort_key=sort_key)
     if any(arg is not None for arg in [
             include_folders,
             exclude_folders,
             include_files,
             exclude_files,
             mask]):
-        listdir = RealDirStructure.filter_items(listdir,
-                                                include_folders=include_folders,
-                                                exclude_folders=exclude_folders,
-                                                include_files=include_files,
-                                                exclude_files=exclude_files,
-                                                regex=regex,
-                                                mask=mask)
+        listdir = RDS.filter_items(listdir,
+                                   include_folders=include_folders,
+                                   exclude_folders=exclude_folders,
+                                   include_files=include_files,
+                                   exclude_files=exclude_files,
+                                   regex=regex,
+                                   mask=mask)
     for i, f in enumerate(listdir):
         name = os.path.basename(f)
         if i == itemlimit:
@@ -1326,10 +1361,12 @@ def fakedir_fromstring(s, start_chars=None, name_chars=None,
         Fake directory corresponding to the input string.
 
     '''
+
+    # for detecting folders
     slashes = ['/', '\\', os.sep]
     joinedslashes = ''.join(slashes)
 
-    byline = s.split('\n')
+    # character sets used for detection of items
     keyboard_chars = (string.ascii_letters + string.digits
                       + string.punctuation)
     filtered = "".join([c for c in keyboard_chars if c not in '/:?"*<>|'])
@@ -1338,77 +1375,121 @@ def fakedir_fromstring(s, start_chars=None, name_chars=None,
     if name_chars is None:
         name_chars = filtered + ' ' + '-'
 
-    names = []
-    headers = []
-    depths = []
+    if header_regex is None:
+        header_regex = '.*?(?=[{}])'.format(start_chars)
 
+    custom_name_regex = True
+    if name_regex is None:
+        name_regex = '[{}]*[/\\\\]*'.format(name_chars)
+        custom_name_regex = False
+
+    # split text into separate lines
+    byline = s.split('\n')
+
+    # holders to indicate attributes of each line
+    names = []       # the folder/file name
+    depths = []      # the depth of the item (the size of the header)
+
+    # MAIN LOOP 1: Pull out the folders/files from the diagram
     for line in byline:
+
+        # skip empty lines
         if not line:
             continue
-        if header_regex is None:
-            header = re.match('.*?(?=[{}])'.format(start_chars), line)
-        else:
-            header = re.match(header_regex, line)
+        
+        # --- DEPTH --- 
+        header = re.match(header_regex, line)
         if header is None:
             continue
         else:
             header = header.group()
         depth = len(header)
-        if name_regex is None:
-            name = re.match('[{}]*[/\\\\]*'.format(name_chars), line[depth:])
-        else:
-            name = re.match(name_regex, line)
+
+        # --- NAME --- 
+        search_str = line if custom_name_regex else line[depth:]
+        name = re.match(name_regex, search_str)
         if name is None:
             continue
         else:
             name = name.group()
+
+        # parse comments in name if requested
         if '#' in name and parse_comments:
             name = re.match('.*?(?=#)', name).group().strip()
         if not name:
             continue
 
-        headers.append(header)
+        # store the attributes
         names.append(name)
         depths.append(depth)
 
-    fakeitems = []
-    superparent = None
-    min_depth = min(depths)
+    # SETUP for constructing fakeitems from extracted data
+    fakeitems = []           # holder for all fakeitems - one for each name/depth
+    superparent = None       # root item
+    min_depth = min(depths)  # smallest depth found (based on header size)
+
+    # Cases where the root isn't clear, so a new parent is created
+    #  1. multiple items existing at min depth
+    #  2. min depth is not the earliest item in the tree
+    # Case 1
     if len([d for d in depths if d == min_depth]) > 1:
         superparent = FakeDir(supername)
+
+    # Case 2
     min_depth_index1 = depths.index(min_depth)
     if any(i > min_depth for i in depths[:min_depth_index1]):
         superparent = FakeDir(supername)
 
-    for i, name in enumerate(names):
+    # MAIN LOOP 2: Assemble the data into a FakeDir
+    for i, (name, depth) in enumerate(zip(names, depths)):
+
+        # detect folder
+        # either ends with slash (assume folder)
+        # or if next item is deeper than current item
         is_folder = False
         if name.strip()[-1] in slashes:
             is_folder = True
         if i < len(names) - 1:
-            if depths[i + 1] > depths[i]:
+            if depths[i + 1] > depth:
                 is_folder = True
+        FakeConstructor = FakeDir if is_folder else FakeFile
 
+        # When making fakeitems, don't include slash
         fmt_name = name.rstrip(joinedslashes)
 
-        if depths[i] == min_depth:
-            if is_folder:
-                fakeitems.append(FakeDir(fmt_name, parent=superparent))
-            else:
-                fakeitems.append(FakeFile(fmt_name, parent=superparent))
+        # If the item is at the min depth, no other parents to consider
+        # so just needs to be added to the tree with the superparent as parent
+        if depth == min_depth:
+            item = FakeConstructor(name=fmt_name, parent=superparent)
+            fakeitems.append(item)
+
+        # Otherwise...
         else:
-            shallower = [d for d in depths[:i] if d < depths[i]]
+
+            # Find any preceding items with lower depth (potential parents of current item)
+            preceding_depths = depths[:i]
+            shallower = [d for d in preceding_depths if d < depth]
+        
+            # of the shallower preceding items, assume the parent is
+            #    - occuring at the maximum depth
+            #    - the last one (later in the string) occuring
             if shallower:
-                max_shallower = max([d for d in depths[:i] if d < depths[i]])
-                parent_index = max(idx for idx, val in enumerate(depths[:i])
-                                   if val == max_shallower)
+                max_shallower = max([d for d in preceding_depths if d < depth])
+                parent_index = max(idx for idx, val in enumerate(preceding_depths) if val == max_shallower)
                 parent = fakeitems[parent_index]
+
+            # item is not the minimum depth, but has no shallower items before it
+            # Weird case, could maybe be a partially copied diagram
+            # Should be captured by Case 2 for superparent logic above
             else:
                 parent = superparent
-            if is_folder:
-                fakeitems.append(FakeDir(fmt_name, parent=parent))
-            else:
-                fakeitems.append(FakeFile(fmt_name, parent=parent))
+            
+            # add item to tree
+            item = FakeConstructor(name=fmt_name, parent=parent)
+            fakeitems.append(item)
 
+    # FINISH UP
+    # Return either the created root, or the detected root
     if superparent is not None:
         return superparent
     else:
